import { Logger } from '@nestjs/common';
import { NestFactory } from '@nestjs/core';
import {
  FastifyAdapter,
  NestFastifyApplication,
} from '@nestjs/platform-fastify';
import { Logger as PinoLogger } from 'nestjs-pino';

import { AppModule } from './app/app.module';
import { LocalConfig, localConfig } from './app/modules/config/local.config';
import { patchTypeOrm } from './app/modules/database/utils/typeorm-patch';

async function bootstrap() {
  await patchTypeOrm();

  const appOpts = (() => {
    const loggerOpts = { bufferLogs: true };
    return {
      forceCloseConnections: true,
      ...loggerOpts,
    };
  })();
  const app = await NestFactory.create<NestFastifyApplication>(
    AppModule,
    new FastifyAdapter(appOpts)
  );
  const globalPrefix = 'api';
  app.setGlobalPrefix(globalPrefix);

  const config = app.get<LocalConfig>(localConfig.KEY);
  const port = config.port || 3000;

  app.useLogger(app.get(PinoLogger));

  await app.listen(port);
  Logger.log(
    `🚀 Application is running on: http://localhost:${port}/${globalPrefix}`
  );
}

void bootstrap();
