import {
  formatFiles,
  generateFiles,
  joinPathFragments,
  names,
  OverwriteStrategy,
  readProjectConfiguration,
  Tree,
} from '@nx/devkit';
import { LibGeneratorSchema } from './schema';

import { libraryGenerator } from '@nx/nest';

export async function libGenerator(tree: Tree, options: LibGeneratorSchema) {
  const projectConfig = readProjectConfiguration(tree, options.project);

  const typeDirectory = (() => {
    switch (options.type) {
      case 'module':
        return 'modules';
      case 'feature':
        return 'features';
    }
  })();

  const libraryName = `${projectConfig.name}-${options.name}`;
  const libraryRoot = joinPathFragments(
    projectConfig.root,
    'src',
    typeDirectory,
    libraryName
  );

  const opts: Parameters<typeof libraryGenerator>[1] = {
    directory: libraryRoot,
    name: libraryName,
    buildable: true,
    linter: 'eslint',
    setParserOptionsProject: true,
    unitTestRunner: 'jest',
    strict: true,
    simpleName: false,
  };
  await libraryGenerator(tree, opts);

  tree.delete(joinPathFragments(libraryRoot, 'src'));
  generateFiles(
    tree,
    joinPathFragments(__dirname, './files/common'),
    libraryRoot,
    { ...opts, ...names(options.name), controller: true, service: true },
    { overwriteStrategy: OverwriteStrategy.Overwrite }
  );
  await formatFiles(tree);
}

export default libGenerator;
